'use strict';

exports.__esModule = true;
exports.inline = inline;
exports.keyframes = keyframes;
exports.fontFace = fontFace;
exports.injectGlobal = injectGlobal;

var _parser = require('./parser');

var _hash = require('./hash');

var _hash2 = _interopRequireDefault(_hash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function extractNameFromProperty(str) {
  var regex = /name\s*:\s*([A-Za-z0-9\-_]+)\s*/gm;
  var match = regex.exec(str);
  if (match) {
    return match[1];
  }
}

function getName(extracted, identifierName, prefix) {
  var parts = [];
  parts.push(prefix);
  if (extracted) {
    parts.push(extracted);
  } else if (identifierName) {
    parts.push(identifierName);
  }
  return parts.join('-');
}

function replaceVarsWithPlaceholders(rules) {
  return rules.map(function (rule) {
    return rule.replace(/var\(--[A-Za-z0-9-_]+-([0-9]+)\)/gm, function (match, p1) {
      return 'xxx' + p1 + 'xxx';
    });
  });
}

function replaceApplyWithPlaceholders(rules) {
  return rules.map(function (rule) {
    return rule.replace(/@apply\s+--[A-Za-z0-9-_]+-([0-9]+)/gm, function (match, p1) {
      return 'xxx' + p1 + 'xxx';
    });
  });
}

function createSrc(strs, name, hash, expressionLength) {
  var hasApply = false;
  var hasVar = false;
  var src = strs.reduce(function (arr, str, i) {
    arr.push(str);
    if (i !== expressionLength) {
      // todo - test for preceding @apply
      var applyMatch = /@apply\s*$/gm.exec(str);
      if (applyMatch) {
        hasApply = true;
        arr.push('--' + name + '-' + hash + '-' + i);
      } else {
        hasVar = true;
        arr.push('var(--' + name + '-' + hash + '-' + i + ')');
      }
    }
    return arr;
  }, []).join('').trim();
  return { src: src, hasApply: hasApply, hasVar: hasVar };
}

function inline(quasi, identifierName, prefix, inlineMode) {
  var strs = quasi.quasis.map(function (x) {
    return x.value.cooked;
  });
  var hash = (0, _hash2.default)([].concat(strs) // todo - add current filename?
  );var name = getName(extractNameFromProperty(strs.join('xxx')), identifierName, prefix);

  var _createSrc = createSrc(strs, name, hash, quasi.expressions.length),
      src = _createSrc.src,
      hasApply = _createSrc.hasApply,
      hasVar = _createSrc.hasVar;

  var rules = (0, _parser.parseCSS)('.' + name + '-' + hash + ' { ' + src + ' }');
  if (inlineMode || hasApply) {
    rules = replaceVarsWithPlaceholders(rules);
  }
  if (hasApply) {
    rules = replaceApplyWithPlaceholders(rules);
  }
  return { hash: hash, name: name, rules: rules, hasApply: hasApply, hasVar: hasVar };
}

function keyframes(quasi, identifierName, prefix) {
  var strs = quasi.quasis.map(function (x) {
    return x.value.cooked;
  });
  var hash = (0, _hash2.default)([].concat(strs));
  var name = getName(extractNameFromProperty(strs.join('xxx')), identifierName, prefix);

  var _createSrc2 = createSrc(strs, name, hash, quasi.expressions.length),
      src = _createSrc2.src,
      hasApply = _createSrc2.hasApply,
      hasVar = _createSrc2.hasVar;

  var rules = (0, _parser.parseCSS)('{ ' + src + ' }', { nested: false });
  if (hasApply) {
    rules = replaceApplyWithPlaceholders(rules);
  }
  if (hasVar) {
    rules = replaceVarsWithPlaceholders(rules);
  }
  return { hash: hash, name: name, rules: rules, hasApply: hasApply, hasVar: hasVar };
}

function fontFace(quasi) {
  var strs = quasi.quasis.map(function (x) {
    return x.value.cooked;
  });

  var _createSrc3 = createSrc(strs, 'name', 'hash', quasi.expressions.length),
      src = _createSrc3.src,
      hasApply = _createSrc3.hasApply,
      hasVar = _createSrc3.hasVar;

  var rules = (0, _parser.parseCSS)('@font-face {' + src + '}');
  if (hasVar) {
    rules = replaceVarsWithPlaceholders(rules);
  }
  return { rules: rules, hasApply: hasApply, hasVar: hasVar };
}

function injectGlobal(quasi) {
  var strs = quasi.quasis.map(function (x) {
    return x.value.cooked;
  });

  var _createSrc4 = createSrc(strs, 'name', 'hash', quasi.expressions.length),
      src = _createSrc4.src,
      hasVar = _createSrc4.hasVar,
      hasApply = _createSrc4.hasApply;

  var rules = (0, _parser.parseCSS)(src);
  if (hasApply) {
    rules = replaceApplyWithPlaceholders(rules);
  }
  if (hasVar) {
    rules = replaceVarsWithPlaceholders(rules);
  }
  return { rules: rules, hasApply: hasApply, hasVar: hasVar };
}